C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set User Defined Energy Scale   _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      SUBROUTINE GRCUSRSETQ
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'
      INCLUDE 'inclm.h'
C...User-defined energy scale.
C
C   The definition is effective only when "ICOUP" or "IFACT" = 6.
C   Users must set their-defined energy scale into "GRCQ" or "GRCFAQ".
C
C...4-momentum in the CM frame of the hard interaction
C   PGRC(i,j)  (i=1,4:(px,py,pz,E)): 4-momentum of j-th particle.
C   PQGRC(i,j) (i=1,4:(px,py,pz,E)): 4-momentum of j-th resonance particle.

      call setscale( igsub, pgrc, grcq, grcfaq )

      RETURN
      END
C------------------------------------------------------------------------
      subroutine setscale( igsub, p, qren, qfac )
      IMPLICIT REAL*8(A-H,O-Z)
      real*8 p(4,10)
      real*8 pvec(4,2)
      parameter ( qmin= 10.d0 )

      qren= qmin
      qfac= qmin

C  Di-boson production processes
      if( ( igsub.ge.120 .and. igsub.lt.150 )
     >     .or. ( igsub.ge.550 .and. igsub.lt.580 ) ) then
         do i=1,4
            pvec(i,1)= p(i,3) +p(i,4)
            pvec(i,2)= p(i,5) +p(i,6)
         enddo
         am1sq= pvec(4,1)**2 -pvec(3,1)**2 -pvec(2,1)**2 -pvec(1,1)**2
         am2sq= pvec(4,2)**2 -pvec(3,2)**2 -pvec(2,2)**2 -pvec(1,2)**2
         qdif= sqrt( ( am1sq +am2sq )/2.d0
     >              +( ( pvec(1,1) -pvec(1,2) )**2
     >                +( pvec(2,1) -pvec(2,2) )**2 )/4.d0 )
         if( qdif.gt.10.d0 ) then
            qren= qdif
            qfac= qdif
         endif
      endif

      return
      end
C------------------------------------------------------------------------
      subroutine scale_fsr( scale )
C  Return the FSR scale.
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'

      scale= GRCFAQ

      RETURN
      END
C------------------------------------------------------------------------
      subroutine scale_fsub( igsub, p, scale )
C  Return the energy scale used in the final-state LLL subtraction.
      IMPLICIT REAL*8(A-H,O-Z)
      real*8 p(4,10)

      call setscale( igsub, p, qren, qfac )
      scale= qfac

      RETURN
      END
C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set User Defined Cut  _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      SUBROUTINE GRCUSRCUT(IUSRCUT)
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'
      INTEGER IUSRCUT
      INTEGER GRCKFCD
      EXTERNAL GRCKFCD
      DOUBLE PRECISION GRCIVMS,GRCDPHI,GRCDRCN
      EXTERNAL GRCIVMS,GRCDPHI,GRCDRCN
C...User-defined kinematic cuts.
C
C   The returned argument is "IUSRCUT".
C   If events pass their cuts, IUSRCUT = 0,
C         not pass             IUSRCUT = 1.
C
C   Users must carefully check the convergence of the BASES iteration 
C   if they define a tight cut using this subroutine.
C
C...4-momentum in the CM frame of the hard interaction
C     PGRC(I,J)  (I=1,4:(px,py,pz,E))  : 4-momentum of J-th particle.
C
C...4-momentum in the Laboratory frame.
C     PLGRC(I,J) (I=1,4:(pT,phi,eta,E)): 4-momentum of J-th (J > 2) 
C                                        final state particle.
C
C...Useful stuff.
C     FUNCTION GRCKFCD(I)   : Output particle identification code.
C     FUNCTION GRCIVMS(I,J) : Output inv. mass between "I" and "J" particles.
C     FUNCTION GRCDPHI(I,J) : Output Delta Phi between "I" and "J" particles.
C     FUNCTION GRCDRCN(I,J) : Output Delta R between "I" and "J" particles.
 
      parameter ( amzmin= 60.d0, amzmax= 99999.d0 )

      IUSRCUT = 0
 
C  Limit the invariant mass of Z/gamma*, to reduce the gamma* contribution.

C  ZW + jets
      if( ( igsub.ge.130 .and. igsub.le.132 )
     >     .or. igsub.eq.552 .or. igsub.eq.557 .or. igsub.eq.558
     >     .or. ( igsub.ge.573 .and. igsub.le.579 ) ) then
         ainvmas= GRCIVMS(3,4)
         if( ainvmas.lt.amzmin .or. ainvmas.gt.amzmax ) IUSRCUT = 1
      endif

C  ZZ + jets
      if( ( igsub.ge.140 .and. igsub.le.142 )
     >     .or. igsub.eq.550 .or. igsub.eq.553 .or. igsub.eq.554
     >     .or. ( igsub.ge.559 .and. igsub.le.565 ) ) then
         ainvmas= GRCIVMS(3,4)
         if( ainvmas.lt.amzmin .or. ainvmas.gt.amzmax ) IUSRCUT = 1
         ainvmas= GRCIVMS(5,6)
         if( ainvmas.lt.amzmin .or. ainvmas.gt.amzmax ) IUSRCUT = 1
      endif

      RETURN
      END
 
C//////////////////////////////////////////
C  Laboratory frame cut inside "FUNC".
C  To be calles when "labcut" = 1.
C//////////////////////////////////////////
      subroutine grclabcut( icut )
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)

      icut= 0

      return
      end
