      subroutine QPsetLd( vlambda )
C  Set the QCD Lambda value to be used in QCDPS.
C  (input) vlambda: Lmbda value in GeV.
C  If "vlambda" > 0, this value is forced to be used.
C  If "vlambda" < 0, Lambda is derived from the alpha_s value given by PDF. 
C  This routine has to be called after calling the QCDPS initialization 
C  routine QPINIT if users want to set a specific value.
C  If calling with vlambda < 0, this routine has to be called after 
C  completing the initialization of PDF.
C  Note that in QCDPS everything is done with the leading-order evaluation of 
C  alpha_s for 5 quark flavors.
C  Aug 23, 2011 S.Odaka
      implicit real*8 (a-h,o-z)
      include 'qpincl.h'
      parameter ( twopi= 6.283185307179586d0 )
C  The energy scale (GeV) at which the alpha_s value is normalized.
      parameter ( fscale= 10.0d0 )

      print *,'QPsetLd: set the QCD Lambda value for QCDPS.'

      if( vlambda.gt.0.d0 ) then
         QCDLd= vlambda
         print '(5X,''Forced to be '',F7.5,'' GeV.'')',QCDLd

      else
         print '(5X,''Derive the value that gives the same alpha_s '',
     >        ''as that given'')'
         print '(5X,''by the chosen PDF at '',F7.2,'' GeV.'')',fscale
         alps= GRCALPS( fscale )
         QCDLd= fscale/exp( twopi/beta/alps )
         print '(5X,''-> Lambda = '',F7.5,'' GeV.'')',QCDLd
      endif

      return
      end
