      subroutine ps2lha_full
C  Copy the event information from PS arrays to the LHA common.
C  The event information must be fully stored in the PS arrays, including 
C  those of the hard interaction and the final-state PS.
C  The information of the produced particles are copied only if they have 
C  a status code of 4 (unevolved radiation), 5 (partons at the end of 
C  evolution), or 10 (unevolved particles from the hard interaction).
C  Oct 27 2009, S. Odaka
C
C  Modified to support QED FSR.
C  Only the photon radiation in the final state is supported.
C  Photons should not branch further.
C  Sep 20 2011, S.O.
      IMPLICIT REAL*8(A-H,O-Z)
      include 'grchad.inc'

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPcolor/icptn(2,10000)
      common/QProot/iroot(10000)
      common/QPinfo/Qps,ipps(3,2)
C  ipps(j,i): j'th pointer for the i'th beam (i = 1,2)
C       j = 1 starting point (beam particle)
C           2 final colliding parton
C           3 end point (usually a radiation)

      do ip=1,NTOP
         iroot(ip)= 0
      enddo

C -- Determine the color flow.

C  The largest color index already used.
      icmax= 0
      do i=1,NUP
         do j=1,2
            if( ICOLUP(j,i).gt.icmax ) icmax= ICOLUP(j,i)
         enddo
      enddo

C  Major branches in the initial-state PS.
C  Backward determination.
      if( ishower.gt.0 ) then
         do ibeam=1,2
            ip= ipps(2,ibeam)
            icptn(1,ip)= ICOLUP(1,ibeam)
            icptn(2,ip)= ICOLUP(2,ibeam)
            iroot(ip)= ibeam
 1          ipr= NLPTN(2,ip)
            if( ipr.gt.0 ) then
               ip0= ip -ipr
               iroot(ip0)= ibeam
               id0= NLPTN(1,ip0)
               if( NLPTN(3,ip0).eq.2 ) then
                  ip1= ip +1
                  iroot(ip1)= ibeam
                  id= NLPTN(1,ip)
                  if( iabs( id ).le.6 ) then
                     if( iabs( id0 ).le.6 ) then
                        if( id.gt.0 ) then
C  q* -> q* g
                           icmax= icmax +1
                           icptn(1,ip0)= icmax
                           icptn(2,ip0)= 0
                           icptn(1,ip1)= icmax
                           icptn(2,ip1)= icptn(1,ip)
                        else
C  qbar* -> qbar* g
                           icmax= icmax +1
                           icptn(1,ip0)= 0
                           icptn(2,ip0)= icmax
                           icptn(1,ip1)= icptn(2,ip)
                           icptn(2,ip1)= icmax
                        endif
                     else
                        if( id.gt.0 ) then
C  g* -> q* qbar
                           icmax= icmax +1
                           icptn(1,ip0)= icptn(1,ip)
                           icptn(2,ip0)= icmax
                           icptn(1,ip1)= 0
                           icptn(2,ip1)= icmax
                        else
C  g* -> qbar* q
                           icmax= icmax +1
                           icptn(1,ip0)= icmax
                           icptn(2,ip0)= icptn(2,ip)
                           icptn(1,ip1)= icmax
                           icptn(2,ip1)= 0
                        endif
                     endif
                  else
                     if( iabs( id0 ).le.6 ) then
                        if( id0.gt.0 ) then
C  q* -> g* q
                           icptn(1,ip0)= icptn(1,ip)
                           icptn(2,ip0)= 0
                           icptn(1,ip1)= icptn(2,ip)
                           icptn(2,ip1)= 0
                        else
C  qbar* -> g* qbar
                           icptn(1,ip0)= 0
                           icptn(2,ip0)= icptn(2,ip)
                           icptn(1,ip1)= 0
                           icptn(2,ip1)= icptn(1,ip)
                        endif
                     else
C  g* -> g* g
                        icmax= icmax +1
                        if( QPRAND(idum).lt.0.5d0 ) then
                           icptn(1,ip0)= icptn(1,ip)
                           icptn(2,ip0)= icmax
                           icptn(1,ip1)= icptn(2,ip)
                           icptn(2,ip1)= icmax
                        else
                           icptn(1,ip0)= icmax
                           icptn(2,ip0)= icptn(2,ip)
                           icptn(1,ip1)= icmax
                           icptn(2,ip1)= icptn(1,ip)
                        endif
                     endif
                  endif
                  ip= ip0
                  goto 1
               endif
            endif
         enddo
      endif

C  Minor branches in the initial state and branches in the final state; 
C  i.e., branches of time-like partons.
      do ip=1,NTOP
         if( NLPTN(6,ip).eq.1 .and. 
     >        icptn(1,ip)+icptn(2,ip).ne.0 ) then
            if( NLPTN(3,ip).eq.1 ) then
               ip1= ip +NLPTN(4,ip)
               icptn(1,ip1)= icptn(1,ip)
               icptn(2,ip1)= icptn(2,ip)
               iroot(ip1)= iroot(ip)
            elseif( NLPTN(3,ip).eq.2 ) then
               id= NLPTN(1,ip)
               ip1= ip +NLPTN(4,ip)
               id1= NLPTN(1,ip1)
               ip2= ip1 +1
               id2= NLPTN(1,ip2)
               if( iabs( id ).le.6 ) then
                  if( iabs( id1 ).le.6 ) then
                     if( id2.eq.21 ) then
                        if( id.gt.0 ) then
C  q* -> q* g
                           icmax= icmax +1
                           icptn(1,ip1)= icmax
                           icptn(2,ip1)= 0
                           icptn(1,ip2)= icptn(1,ip)
                           icptn(2,ip2)= icmax
                        else
C  qbar* -> qbar* g
                           icmax= icmax +1
                           icptn(1,ip1)= 0
                           icptn(2,ip1)= icmax
                           icptn(1,ip2)= icmax
                           icptn(2,ip2)= icptn(2,ip)
                        endif
                     else
C  q*(qbar*) -> q*(qbar*) gam
                        icptn(1,ip1)= icptn(1,ip)
                        icptn(2,ip1)= icptn(2,ip)
                        icptn(1,ip2)= 0
                        icptn(2,ip2)= 0
                     endif
                  else
                     if( id.gt.0 ) then
C  q* -> g* q
                        icmax= icmax +1
                        icptn(1,ip1)= icptn(1,ip)
                        icptn(2,ip1)= icmax
                        icptn(1,ip2)= icmax
                        icptn(2,ip2)= 0
                     else
C  qbar* -> g* qbar
                        icmax= icmax +1
                        icptn(1,ip1)= icmax
                        icptn(2,ip1)= icptn(2,ip)
                        icptn(1,ip2)= 0
                        icptn(2,ip2)= icmax
                     endif
                  endif
               else
                  if( iabs( id1 ).le.6 ) then
                     if( id1.gt.0 ) then
C  g* -> q* qbar
                        icptn(1,ip1)= icptn(1,ip)
                        icptn(2,ip1)= 0
                        icptn(1,ip2)= 0
                        icptn(2,ip2)= icptn(2,ip)
                     else
C  g* -> qbar* q
                        icptn(1,ip1)= 0
                        icptn(2,ip1)= icptn(2,ip)
                        icptn(1,ip2)= icptn(1,ip)
                        icptn(2,ip2)= 0
                     endif
                  else
C  g* -> g* g
                     icmax= icmax +1
                     if( QPRAND(idum).lt.0.5d0 ) then
                        icptn(1,ip1)= icptn(1,ip)
                        icptn(2,ip1)= icmax
                        icptn(1,ip2)= icmax
                        icptn(2,ip2)= icptn(2,ip)
                     else
                        icptn(1,ip1)= icmax
                        icptn(2,ip1)= icptn(2,ip)
                        icptn(1,ip2)= icptn(1,ip)
                        icptn(2,ip2)= icmax
                     endif
                  endif
               endif
               iroot(ip1)= iroot(ip)
               iroot(ip2)= iroot(ip)
            endif
         endif
      enddo

C -- Copy the particle info to the LHA common.
      SCALUP= Qps

C  Initial state.
      if( ishower.gt.0 ) then
C  If the initial-state PS has been applied, ...
C  Initial partons.
         do i=1,2
            ip= ipps(1,i) +1
            IDUP(i)= NLPTN(1,ip)
            ISTUP(i)= -1
            MOTHUP(1,i)= 0
            MOTHUP(2,i)= 0
            ICOLUP(1,i)= icptn(1,ip)
            ICOLUP(2,i)= icptn(2,ip)
            do k=1,4
               PUP(k,i)= PLPTN(k+3,ip)
            enddo
            PUP(5,i)= ptnmas( IDUP(i) )
            VTIMUP(i)= 0.d0
            SPINUP(i)= 0.d0
         enddo
         NUP= 2
C  Initial-state radiation: fix the mass here.
         do ibeam=1,2
            do ip=1,NTOP
               ist= NLPTN(5,ip)
               if( iroot(ip).eq.ibeam .and. 
     >              ( ist.eq.4 .or. ist.eq.5 ) ) then
                  NUP= NUP +1
                  IDUP(NUP)= NLPTN(1,ip)
                  ISTUP(NUP)= 1
                  MOTHUP(1,NUP)= ibeam
                  MOTHUP(2,NUP)= 0
                  ICOLUP(1,NUP)= icptn(1,ip)
                  ICOLUP(2,NUP)= icptn(2,ip)
                  PUP(5,NUP)= ptnmas( IDUP(NUP) )
                  do k=1,3
                     PUP(k,NUP)= PLPTN(k+3,ip)
                  enddo
                  PUP(4,NUP)= sqrt( PUP(1,NUP)**2 +PUP(2,NUP)**2
     >                 +PUP(3,NUP)**2 +PUP(5,NUP)**2 )
                  VTIMUP(NUP)= 0.d0
                  SPINUP(NUP)= 0.d0
               endif
            enddo
         enddo
      else
C  Already recorded info is kept if the initial-state PS is not applied.
         NUP= 2
      endif

C  Final state.
      do ip=1,NTOP
C  iroot > 0 for initial-state radiation.
C         ist= NLPTN(5,ip)
C         if( iroot(ip).le.0 .and. 
C     >        ( ist.eq.10 .or. ist.eq.4 .or. ist.eq.5 ) ) then
C  Fix on 7/1/2010: S.O.
         if( iroot(ip).le.0 .and. 
     >        ( NLPTN(5,ip).eq.10 .or. NLPTN(3,ip).eq.0  ) ) then
            NUP= NUP +1
            IDUP(NUP)= NLPTN(1,ip)
            if( NLPTN(3,ip).eq.0 ) then
               ISTUP(NUP)= 1
            else
               ISTUP(NUP)= 2
C  Use iroot to remember the position of decaying particles.
C  Negative numbers are used in order to avoid confusion.
               do i=ip+1,NTOP
                  if( i-NLPTN(2,i).eq.ip ) then
                     iroot(i)= -NUP
                  endif
               enddo
            endif
            if( iroot(ip).eq.0 ) then
               MOTHUP(1,NUP)= 1
               MOTHUP(2,NUP)= 2
            else
C  Decay products; i.e., a negative number is set to iroot.
               MOTHUP(1,NUP)= -iroot(ip)
               MOTHUP(2,NUP)= 0
            endif
            ICOLUP(1,NUP)= icptn(1,ip)
            ICOLUP(2,NUP)= icptn(2,ip)
C  Fix the mass of quarks and gluons.
            if( iabs( IDUP(NUP) ).le.6 .or. IDUP(NUP).eq.21 ) then
               PUP(5,NUP)= ptnmas( IDUP(NUP) )
               do k=1,3
                  PUP(k,NUP)= PLPTN(k+3,ip)
               enddo
               PUP(4,NUP)= sqrt( PUP(1,NUP)**2 +PUP(2,NUP)**2
     >              +PUP(3,NUP)**2 +PUP(5,NUP)**2 )
            else
C  Already recorded info is used for other particles.
               do k=1,4
                  PUP(k,NUP)= PLPTN(k+3,ip)
               enddo
               amsq= PUP(4,NUP)**2 -PUP(3,NUP)**2
     >              -PUP(2,NUP)**2 -PUP(1,NUP)**2
               if( amsq.lt.0.d0 ) amsq= 0.d0
               PUP(5,NUP)= sqrt( amsq )
            endif
            VTIMUP(NUP)= 0.d0
            SPINUP(NUP)= 0.d0
         endif
      enddo

C -- Fine adjust of the momenta to ensure exact energy/momentum conservation.

C  pt correction:
C  Though the total pt of produced particles must be zero, there may be a 
C  small pt caused by calculation errors.
C  Such pt is removed by adjusting the momenta of directly produced particles.
      pxfin= 0.d0
      pyfin= 0.d0
      pdir= 0.d0
      do i=3,NUP
         if( ISTUP(i).eq.1 ) then
            pxfin= pxfin +PUP(1,i)
            pyfin= pyfin +PUP(2,i)
C  Momentum sum of directly produced particles.
            if( MOTHUP(1,i).le.2 .and. MOTHUP(2,i).le.2 ) then
               pdir= pdir +sqrt( PUP(1,i)**2 +PUP(2,i)**2 +PUP(3,i)**2 )
            endif
         endif
      enddo
      ptfin= sqrt( pxfin**2 +pyfin**2 )
C  Correct if the remaining pt is greater than 1 MeV.
      if( ptfin.gt.1.d-3 ) then
C  The pt should be small compared to the momentum sum.
         if( pdir/ptfin.gt.1.d1 ) then
            if( pdir/ptfin.lt.1.d3 ) then
               print *,'ps2lha_full: pt discrepancy is rather large.'
               ratio= ptfin/pdir
               print *,'  ptfin ',ptfin,' pdir ',pdir,' ratio ',ratio
            endif
            do i=3,NUP
               if( ISTUP(i).eq.1 .and. 
     >              MOTHUP(1,i).le.2 .and. MOTHUP(2,i).le.2 ) then
                  factor= sqrt( PUP(1,i)**2 +PUP(2,i)**2
     >                 +PUP(3,i)**2 )/pdir
                  PUP(1,i)= PUP(1,i) -factor*pxfin
                  PUP(2,i)= PUP(2,i) -factor*pyfin
                  PUP(4,i)= sqrt( PUP(1,i)**2 +PUP(2,i)**2
     >                 +PUP(3,i)**2 +PUP(5,i)**2 )
               endif
            enddo
         else
            print *,'ps2lha_full: pt discrepancy is too large.'
            print *,'  Maybe, a bug in the PS kinematics.'
            print *,'  Event dump:'
            call EUPdump
            ratio= ptfin/pdir
            print *,'  ptfin ',ptfin,' pdir ',pdir,' ratio ',ratio
            stop
         endif
      endif

C  pz and E correction:
C  The pz and E of the initial partons are adjusted to match the final state.
      pzfin= 0.d0
      efin= 0.d0
      do i=3,NUP
         if( ISTUP(i).eq.1 ) then
            pzfin= pzfin +PUP(3,i)
            efin= efin +PUP(4,i)
         endif
      enddo
      shat= efin**2 -pzfin**2
      PUP(5,1)= ptnmas( IDUP(1) )
      PUP(5,2)= ptnmas( IDUP(2) )
      amsq1= PUP(5,1)**2
      amsq2= PUP(5,2)**2
      pcmsq= ( ( shat -amsq1 -amsq2 )**2/4.d0 -amsq1*amsq2 )/shat
      pcm= sqrt( pcmsq )
      beta= pzfin/efin
      gamma= efin/sqrt( shat )
      if( PUP(3,1).gt.PUP(3,2) ) then
         ecm= sqrt( amsq1 +pcmsq )
         PUP(3,1)= gamma*( pcm +beta*ecm )
         PUP(4,1)= gamma*( ecm +beta*pcm )
         ecm= sqrt( amsq2 +pcmsq )
         PUP(3,2)= gamma*( -pcm +beta*ecm )
         PUP(4,2)= gamma*( ecm -beta*pcm )
      else
         ecm= sqrt( amsq1 +pcmsq )
         PUP(3,1)= gamma*( -pcm +beta*ecm )
         PUP(4,1)= gamma*( ecm -beta*pcm )
         ecm= sqrt( amsq2 +pcmsq )
         PUP(3,2)= gamma*( pcm +beta*ecm )
         PUP(4,2)= gamma*( ecm +beta*pcm )
      endif

      return
      end

      subroutine QPDUMP_full
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPcolor/icptn(2,10000)
      common/QProot/iroot(10000)

      WRITE(6,101)NTOP
 101  FORMAT(' LIST OF OUTPUT, # OF LISTS=',I5)
      WRITE(6,102)
 102  FORMAT(9X,' X ',6X,'Q2',8X,'PT2',8X,'PX',8X,'PY',8X,
     &  'PZ',8X,'E',28X,'Wgt',7X,'NLPTN(I,*)=',23X,'color   root')


      DO 110 I =1,NTOP
       WRITE(6,103)I,(PLPTN(K,I),K=1,10),
     .(NLPTN(K,I),K=1,10) ,icptn(1,I),icptn(2,I),iroot(I)
 110  CONTINUE

 103  FORMAT(' ',I3,2X,10(G9.2,1X),I6,9I3,2X,2I4,2X,I3)
 
      return
      end

*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine lha2ps
C  Copy hard interaction information from the LHA common to PS arrays.
C  The events must be tentatively defined in the CM frame of the hard 
C  interaction in the PS array in order to apply a final-state PS.
      IMPLICIT REAL*8(A-H,O-Z)
      include 'grchad.inc'

C----------------- W + jets ---------------
      if( igsub.eq.100 .or. igsub.eq.421 ) then
         call l2p_proc( 1, 0 )
      elseif( igsub.eq.101 .or. 
     >     ( igsub.ge.422 .and. igsub.le.423 ) ) then
         call l2p_proc( 1, 1 )
      elseif( igsub.eq.102 .or. 
     >     ( igsub.ge.424 .and. igsub.le.430 ) ) then
         call l2p_proc( 1, 2 )
      elseif( igsub.eq.103 .or. 
     >     ( igsub.ge.431 .and. igsub.le.438 ) ) then
         call l2p_proc( 1, 3 )
      elseif( igsub.eq.104 .or. 
     >     ( igsub.ge.439 .and. igsub.le.451 ) ) then
         call l2p_proc( 1, 4 )
C----------------- Z + jets ---------------
      elseif( igsub.eq.110 .or. igsub.eq.500 ) then
         call l2p_proc( 1, 0 )
      elseif( igsub.eq.111 .or. 
     >     ( igsub.ge.501 .and. igsub.le.502 ) ) then
         call l2p_proc( 1, 1 )
      elseif( igsub.eq.112 .or. 
     >     ( igsub.ge.503 .and. igsub.le.509 ) ) then
         call l2p_proc( 1, 2 )
      elseif( igsub.eq.113 .or. 
     >     ( igsub.ge.510 .and. igsub.le.517 ) ) then
         call l2p_proc( 1, 3 )
      elseif( igsub.eq.114 .or. 
     >     ( igsub.ge.518 .and. igsub.le.525 ) ) then
         call l2p_proc( 1, 4 )
C----------------- WW + jets ---------------
      elseif( igsub.eq.120 .or. igsub.eq.551 ) then
         call l2p_proc( 2, 0 )
      elseif( igsub.eq.121 .or. 
     >     ( igsub.ge.555 .and. igsub.le.556 ) ) then
         call l2p_proc( 2, 1 )
      elseif( igsub.eq.122 .or. 
     >     ( igsub.ge.566 .and. igsub.le.572 ) ) then
         call l2p_proc( 2, 2 )
C----------------- ZW + jets ---------------
      elseif( igsub.eq.130 .or. igsub.eq.552 ) then
         call l2p_proc( 2, 0 )
      elseif( igsub.eq.131 .or. 
     >     ( igsub.ge.557 .and. igsub.le.558 ) ) then
         call l2p_proc( 2, 1 )
      elseif( igsub.eq.132 .or. 
     >     ( igsub.ge.573 .and. igsub.le.579 ) ) then
         call l2p_proc( 2, 2 )
C----------------- ZZ + jets ---------------
      elseif( igsub.eq.140 .or. igsub.eq.550 ) then
         call l2p_proc( 2, 0 )
      elseif( igsub.eq.141 .or. 
     >     ( igsub.ge.553 .and. igsub.le.554 ) ) then
         call l2p_proc( 2, 1 )
      elseif( igsub.eq.142 .or. 
     >     ( igsub.ge.559 .and. igsub.le.565 ) ) then
         call l2p_proc( 2, 2 )
C----------------- AA + jets ---------------
      elseif( igsub.eq.150 .or. igsub.eq.600 ) then
         call l2p_proc( 2, 0 )
      elseif( igsub.eq.151 .or. 
     >     ( igsub.ge.601 .and. igsub.le.602 ) ) then
         call l2p_proc( 2, 1 )
C----------------- A + jets ----------------
      elseif( igsub.eq.190 .or. 
     >     ( igsub.ge.610 .and. igsub.le.611 ) ) then
         call l2p_proc( 1, 1 )
C----------------- QCD jets ----------------
      elseif( igsub.eq.182 .or. 
     >     ( igsub.ge.301 .and. igsub.le.308 ) ) then
         call l2p_proc( 0, 2 )
C-------------------------------------------
      else
         print *,'lha2ps: does not support the process ',igsub
         stop
      endif

      return
      end

      subroutine l2p_proc( nhvy, njet )
C  For those events having "nhvy" heavy particles plus "njet" jets.
      IMPLICIT REAL*8(A-H,O-Z)
C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPcolor/icptn(2,10000)
      common/l2p_copy/ips_copy(MAXNUP)

      do i=1,NUP
         ips_copy(i)= 0
      enddo

      beta= ( PUP(3,1) +PUP(3,2) )/( PUP(4,1) +PUP(4,2) )
      gamma= 1.d0/sqrt( 1.d0 -beta**2 )
      ilha= 2

C  non-showering particles
      if( nhvy.gt.0 ) then
         do i=1,nhvy
            NTOP= NTOP +1
            ilha= ilha +1
            PLPTN(3,NTOP)= PUP(1,ilha)**2 +PUP(2,ilha)**2
            PLPTN(4,NTOP)= PUP(1,ilha)
            PLPTN(5,NTOP)= PUP(2,ilha)
            PLPTN(6,NTOP)= gamma*( PUP(3,ilha) -beta*PUP(4,ilha) )
            PLPTN(7,NTOP)= gamma*( PUP(4,ilha) -beta*PUP(3,ilha) )
            PLPTN(10,NTOP)= 1.d0

            NLPTN(1,NTOP)= IDUP(ilha)
            NLPTN(2,NTOP)= 0
            NLPTN(3,NTOP)= 0
            NLPTN(4,NTOP)= 0
            NLPTN(5,NTOP)= 10
            NLPTN(6,NTOP)= 1
            NLPTN(7,NTOP)= 0
            NLPTN(8,NTOP)= 0

            icptn(1,NTOP)= ICOLUP(1,ilha)
            icptn(2,NTOP)= ICOLUP(2,ilha)

            ips_copy(ilha)= NTOP
         enddo
      endif

C  jets
      if( njet.gt.0 ) then
         do ijet=1,njet
            NTOP= NTOP +1
            ilha= ilha +1
            PLPTN(1,NTOP)= 1.d0
            PLPTN(3,NTOP)= PUP(1,ilha)**2 +PUP(2,ilha)**2
            PLPTN(4,NTOP)= PUP(1,ilha)
            PLPTN(5,NTOP)= PUP(2,ilha)
            PLPTN(6,NTOP)= gamma*( PUP(3,ilha) -beta*PUP(4,ilha) )
            PLPTN(7,NTOP)= gamma*( PUP(4,ilha) -beta*PUP(3,ilha) )
            PLPTN(10,NTOP)= 1.d0

            NLPTN(1,NTOP)= IDUP(ilha)
            NLPTN(2,NTOP)= 0
            NLPTN(3,NTOP)= 0
            NLPTN(4,NTOP)= 0
            NLPTN(5,NTOP)= 11
            NLPTN(6,NTOP)= 1
            NLPTN(7,NTOP)= 0
            NLPTN(8,NTOP)= 0

            icptn(1,NTOP)= ICOLUP(1,ilha)
            icptn(2,NTOP)= ICOLUP(2,ilha)

            ips_copy(ilha)= NTOP
         enddo
      endif

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine lha2ps_supp( iphard )
C  Copy supplementary event information (e.g., info of decay products) 
C  from the LHA common to PS arrays.
C  A bug fix on 12 July 2010. S.O.
      IMPLICIT REAL*8(A-H,O-Z)

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPcolor/icptn(2,10000)
      common/l2p_copy/ips_copy(MAXNUP)

      real*8 bdir(3)
      real*8 ptemp(4)

      beta= ( PUP(3,1) +PUP(3,2) )/( PUP(4,1) +PUP(4,2) )
      gfact= 1.d0/sqrt( 1.d0 -beta**2 )

C  Look for decaying particles in the LHA event record.
      do ipmlha=3,NUP
         if( ISTUP(ipmlha).eq.2 ) then
            ipmps= ips_copy(ipmlha)

C  Copy the info of decay products.
            ncopy= 0
            do ilha=ipmlha+1,NUP
               if( MOTHUP(1,ilha).eq.ipmlha .or. 
     >              MOTHUP(2,ilha).eq.ipmlha ) then
                  ncopy= ncopy +1
                  ips= NTOP +ncopy
                  PLPTN(4,ips)= PUP(1,ilha)
                  PLPTN(5,ips)= PUP(2,ilha)
                  PLPTN(6,ips)= gfact*( PUP(3,ilha) -beta*PUP(4,ilha) )
                  PLPTN(7,ips)= gfact*( PUP(4,ilha) -beta*PUP(3,ilha) )
                  PLPTN(10,ips)= 1.d0
                  NLPTN(1,ips)= IDUP(ilha)
                  NLPTN(2,ips)= ips -ipmps
                  NLPTN(3,ips)= 0
                  NLPTN(4,ips)= 0
                  NLPTN(5,ips)= 10
                  NLPTN(6,ips)= 1
                  NLPTN(7,ips)= 0
                  NLPTN(8,ips)= 0
                  icptn(1,ips)= ICOLUP(1,ilha)
                  icptn(2,ips)= ICOLUP(2,ilha)
                  ips_copy(ilha)= ips
               endif
            enddo

            NLPTN(3,ipmps)= ncopy
            NLPTN(4,ipmps)= NTOP +1 -ipmps

C  Correct the momenta because the mother particle momentum may have been 
C  altered by PS.
            do j=1,4
               ptemp(j)= 0.d0
            enddo
            do i=1,ncopy
               ip= NTOP +i
               do j=1,4
                  ptemp(j)= ptemp(j) +PLPTN(3+j,ip)
               enddo
            enddo
C  Boost to the CM frame (no rotation).
            call bstcm1( ptemp, bdir, gamma, phi, theta )
            phi= 0.d0
            theta= 0.d0
            do i=1,ncopy
               ip= NTOP +i
               iptemp= NTOP +ncopy +i
               call bstfw( PLPTN(4,ip), 
     >              bdir, gamma, phi, theta, PLPTN(4,iptemp) )
            enddo
C  Boost to the lab frame in the PS array.
            call bstcm1( PLPTN(4,ipmps), bdir, gamma, phi, theta )
            phi= 0.d0
            theta= 0.d0
            do i=1,ncopy
               ip= NTOP +i
               iptemp= NTOP +ncopy +i
               call bstbk( PLPTN(4,iptemp), 
     >              bdir, gamma, phi, theta, PLPTN(4,ip) )
            enddo
C  Clean up.
            do i=1,ncopy
               iptemp= NTOP +ncopy +i
               do j=4,7
                  PLPTN(j,iptemp)= 0.d0
               enddo
            enddo

C  Finalization.
            NTOP= NTOP +ncopy

         endif
      enddo

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine fixPSCol
C  Fix the color-flow information in the PS array.
C  The information for the initial-state partons is not corrected since 
C  the pointers "ips_copy(i)" are not set for them.
C  However, this is not a ploblem because the initial-state information 
C  is directly referred to in the subroutine "ps2lha_full".
      IMPLICIT REAL*8(A-H,O-Z)

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      common/QPcolor/icptn(2,10000)
      common/l2p_copy/ips_copy(MAXNUP)

      do ilha=1,NUP
         ips= ips_copy(ilha)
         if( ips.gt.0 ) then
            do i=1,2
               icptn(i,ips)= ICOLUP(i,ilha)
            enddo
         endif
      enddo

      return
      end
